
/* handler for passing info for file areas to the db server */

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>

#include "db.h"

extern struct ll_files *files_config;
extern struct ll_protos *proto_config;
extern BBS_Config *config;

/* internal functions */
DBM *open_list(FILE_Area *area);
void read_proto_config(void);

int db_file_area(FILE_Area *area, char *name) {

     struct ll_files *cur;

     cur = files_config;

     /* look for the area name */
     while((cur != NULL) && (strcasecmp(name, cur->data.name))) 
	  cur = cur->next;

     /* never found the area they wanted */
     if(cur == NULL)
	  return FAILURE;

     memcpy(area, &(cur->data), sizeof(FILE_Area));
     return SUCCESS;
}

int db_file_info(FILE_Info *info) {

     FILE_Area cur_area;
     DBM *list;
     datum key;
     datum data;
     char lookup[90];

     /* find the area */
     if(db_file_area(&cur_area, info->area) != 0)
	  return FAILURE;
     
     /* open the file listings */
     if((list = open_list(&cur_area)) == NULL)
	  return FAILURE;

     
     key.dptr = info->name;
     key.dsize = strlen(info->name);

     data = dbm_fetch(list, key);
     
     /* found this file? */
     if(data.dptr == NULL) {
	  dbm_close(list);
	  return FAILURE;
     }

     memcpy(info, data.dptr, sizeof(FILE_Info));

     key.dsize = sprintf(lookup, "%s..desc", info->name);
     key.dptr = lookup;
     
     data = dbm_fetch(list, key);
     dbm_close(list);

     /* description there? if not, return an empty string (a couple of
      nulls should to for strlen() to show 0 */
     if(data.dptr == NULL) {
	  info->desc = malloc(2);
	  info->desclen = 2;
	  info->desc[0] = 0;
	  info->desc[1] = 0;
	  return SUCCESS;
     }

     /* copy in the description, and take care to null terminate it. */
     info->desc = malloc(data.dsize+1);
     memcpy(info->desc, data.dptr, data.dsize);
     info->desc[data.dsize] = 0;
     info->desclen = data.dsize+1;

     return SUCCESS;
}

FILE_Info *db_file_list(char *area, int *len) {

     DBM *list;
     datum key;
     datum data;
     FILE_Area cur_area;
     FILE_Info *ret = NULL;

     (*len) = 0;

     /* get area... */
     if(db_file_area(&cur_area, area) != SUCCESS)
	  return NULL;

     if((list = open_list(&cur_area)) == NULL)
	  return NULL;

     key = dbm_firstkey(list);
     while(key.dptr != NULL) {

	  if(strstr(key.dptr, "..desc") != NULL) {
	       key = dbm_nextkey(list);
	       continue;
	  }
	  ret = realloc(ret, sizeof(FILE_Info)*((*len)+1));
	  data = dbm_fetch(list, key);

	  memcpy(&(ret[(*len)]), data.dptr, data.dsize);
	  (*len)++;
	  key = dbm_nextkey(list);
     }

     dbm_close(list);

     return ret;
}

/* adds knowledge of a file and it's description into the area. */

int db_file_add(FILE_Info *info) {

     FILE_Area cur_area;
     char lookup[90];
     DBM *list;
     datum key;
     datum data;

     if(db_file_area(&cur_area, info->area) != SUCCESS)
	  return FAILURE;

     if((list = open_list(&cur_area)) == NULL)
	  return FAILURE;

     key.dptr = info->name;
     key.dsize = strlen(info->name);
     data.dptr = (void *)info;
     data.dsize = sizeof(FILE_Info);

     dbm_store(list, key, data, DBM_REPLACE);
     
     key.dsize = sprintf(lookup, "%s..desc", info->name);
     key.dptr = lookup;
     data.dptr = info->desc;
     data.dsize = info->desclen;

     dbm_store(list, key, data, DBM_REPLACE);

     dbm_close(list);
     
     return SUCCESS;
}

int db_file_clear(FILE_Info *info) {

     FILE_Area cur_area;
     char lookup[90];

     DBM *list;
     datum key;
     datum data;

     if(db_file_area(&cur_area, info->area) != SUCCESS)
	  return FAILURE;
     
     if((list = open_list(&cur_area)) == NULL)
	  return FAILURE;
     
     key.dptr = info->name;
     key.dsize = strlen(info->name);

     dbm_delete(list, key);

     key.dsize = sprintf(lookup, "%s..desc", info->name);
     key.dptr = lookup;

     dbm_delete(list, key);

     dbm_close(list);

     return SUCCESS;

}

/* send back the list of protocols */

int db_file_protos(FILE_Proto **list, int *len) {

     struct ll_protos *cur;

     /* any external protocols? */
     if(proto_config == NULL)
	  return FAILURE;

     cur = proto_config;
     (*len) = 0;
     (*list) = NULL;

     while(cur != NULL) {
	  (*list) = realloc((*list), sizeof(FILE_Proto)*((*len)+1));
	  memcpy(&((*list)[(*len)]), &(cur->data), sizeof(FILE_Proto));
	  cur = cur->next;
	  (*len)++;
     }

     return SUCCESS;

}


/* and now, the generic stuff */

     
int read_file_config(void) {

     FILE *fconfig;
     FILE *list;
     char listpath[160];
     char areaname[80];
     char confpath[160];
     char *field;
     char *value;
     int total = 0;

     struct ll_files *cur;
     struct ll_files *prev = NULL;

     sprintf(listpath, "%s/files/areas", config->base_dir);
     if((list = fopen(listpath, "r")) == NULL) {
	  fprintf(stderr, "Unable to open %s\n", listpath);
	  exit(127);
     }

     files_config = calloc(1, sizeof(struct ll_files));
     cur = files_config;

     field = alloca(80);

     while(fgets(areaname, 80, list) != NULL) {

	  /* junk lines starting with garbage */
	  if((areaname[0] < 33) || (areaname[0] > 126))
	       continue;

	  /* clean up trailing newline */
	  areaname[strlen(areaname)-1] = 0;

	  sprintf(confpath, "%s/files/%s/config", config->base_dir,
		  areaname);
	  if((fconfig = fopen(confpath, "r")) == NULL) {
	       printf("Couldn't open %s, skipping\n", confpath);
	       continue;
	  }

	  while(fgets(field, 80, fconfig) != NULL) {
	       char *fullline = field;

	       field = strsep(&fullline, " ");
	       value = strsep(&fullline, "\n");

	       if(!strncmp(field, "name", 4)) 
		    strcpy(cur->data.name, value);
	       else if(!strncmp(field, "path", 4))
		    strcpy(cur->data.path, value);
	       else if(!strncmp(field, "desc", 4))
		    strcpy(cur->data.desc, value);
	       else if(!strncmp(field, "read", 4))
		    cur->data.read = atoi(value);
	       else if(!strncmp(field, "write", 4))
		    cur->data.write = atoi(value);
	       else 
		    printf("Unknown area config directive %s\n",
			   field);
	  }

	  /* alright, we've gone through the whole config file */
	  fclose(fconfig);
	  
	  prev = cur;
	  cur->next = calloc(1, sizeof(struct ll_files));
	  cur = cur->next;
	  total++;
     }

     printf("%d file areas configured.\n", total);

     prev->next = NULL;

     free(cur);

     read_proto_config();
     
     return SUCCESS;

}
     
DBM *open_list(FILE_Area *area) {

     DBM *ret;
     char dbname[160];
     
     sprintf(dbname, "%s/files/%s.db", config->base_dir, area->name);
     ret = dbm_open(dbname, O_RDWR | O_CREAT | O_APPEND, S_IRUSR |
		    S_IWUSR | S_IRGRP | S_IWGRP);

     if(ret == NULL)
	  fprintf(stderr, "Failed to open %s\n", dbname);

     return ret;
}

/* external protocol config reader. */

void read_proto_config(void) {

     FILE *protconf;
     char *field;
     char *value;
     char confpath[160];
     int total = 0;

     struct ll_protos *cur;

     sprintf(confpath, "%s/files/protos", config->base_dir);

     /* no external protocols */
     if((protconf = fopen(confpath, "r")) == NULL) {
	  printf("Couldn't open %s\n", confpath);
	  printf("No external protocols configured\n");
	  proto_config = NULL;
	  return;
     }

     /* if we could open the file, we'll assume that there's at least
	one protocol configured */

     cur = calloc(sizeof(struct ll_protos), 1);
     proto_config = cur;
     cur->next = NULL;


     field = alloca(160);

     while(fgets(field, 160, protconf) != NULL) {
	  char *fullline = field;

	  field = strsep(&fullline, " ");
	  value = strsep(&fullline, "\n");
	  
	  if(!strcmp(field, "proto")) {
	       /* are we still on the first one? */
	       if(cur->data.protid != 0) {
	       cur->next = calloc(1, sizeof(struct ll_protos));
	       cur = cur->next;
	       cur->next = NULL;
	       }
	       strcpy(cur->data.name, value);
	       total++;
	  }
	  else if(!strcmp(field, "send"))
	       strcpy(cur->data.scmd, value);
	  else if(!strcmp(field, "recv"))
	       strcpy(cur->data.rcmd, value);
	  else if(!strcmp(field, "scmdline"))
	       strcpy(cur->data.scmdline, value);
	  else if(!strcmp(field, "rcmdline"))
	       strcpy(cur->data.rcmdline, value);
	  else if(!strcmp(field, "id"))
	       cur->data.protid = atoi(value);
	  else if(!strcpy(field, "batch"))
	       cur->data.batch = atoi(value);
	  else
	       printf("Unknown protocol directive %s\n", field);

	  continue;
     }

     printf("%d external protocols configured.\n", total);
     fclose(protconf);
     
     return;

}
